@@ -1,4 +1,4 @@
-# Copyright (C) 2018-2021 Rocky Bernstein <rocky@gnu.org>
+# Copyright (C) 2018-2022 Rocky Bernstein <rocky@gnu.org>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -43,18 +43,19 @@ author_email = "rb@dustyfeet.com"
 entry_points = {
     "console_scripts": [
         "decompyle3=decompyle3.bin.decompile:main_bin",
+        "decompyle3-code=decompyle3.bin.decompile_code_type:main",
         "py3disassemble=decompyle3.bin.py3disassemble:main",
     ]
 }
 ftp_url = None
-install_requires = ["spark-parser >= 1.8.9, < 1.9.0", "xdis >= 6.0.2,<6.1.0"]
+install_requires = ["click", "spark-parser >= 1.8.9, < 1.9.0", "xdis >= 6.0.4,<6.1.0"]
 
 license = "GPL3"
 mailing_list = "python-debugger@googlegroups.com"
 modname = "decompyle3"
 py_modules = None
 short_desc = "Python cross-version byte-code decompiler"
-web = "https://github.com/rocky/python-decompyle3/"
+web = "https://github.com/rocky/python-decompile3/"
 
 # tracebacks in zip files are funky and not debuggable
 zip_safe = True
