@@ -1,4 +1,4 @@
-# Copyright (C) 2018-2021 Rocky Bernstein <rocky@gnu.org>
+# Copyright (C) 2018-2022 Rocky Bernstein <rocky@gnu.org>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -13,28 +13,34 @@
 #  You should have received a copy of the GNU General Public License
 #  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import datetime, py_compile, os, sys
-from typing import Any, Tuple
+import ast, datetime, py_compile, os, sys
+import os.path as osp
+import subprocess
+import tempfile
+from typing import Any, Optional, Tuple
 from xdis import iscode, load_module
-from xdis.version_info import version_tuple_to_str
+from xdis.version_info import (
+    IS_PYPY,
+    PYTHON_VERSION_TRIPLE,
+    version_tuple_to_str,
+)
 
-from decompyle3 import IS_PYPY, PYTHON_VERSION_TRIPLE
 from decompyle3.disas import check_object_path
 from decompyle3.semantics import pysource
-from decompyle3.parsers import ParserError
+from decompyle3.parsers.parse_heads import ParserError
 from decompyle3.version import __version__
 
 # from decompyle3.linenumbers import line_number_mapping
 
-from decompyle3.semantics.pysource import code_deparse
+from decompyle3.semantics.pysource import code_deparse, PARSER_DEFAULT_DEBUG
 from decompyle3.semantics.fragments import code_deparse as code_deparse_fragments
 from decompyle3.semantics.linemap import deparse_code_with_map
 
 
 def _get_outstream(outfile: str) -> Any:
-    dir = os.path.dirname(outfile)
+    dir = osp.dirname(outfile)
     failed_file = outfile + "_failed"
-    if os.path.exists(failed_file):
+    if osp.exists(failed_file):
         os.remove(failed_file)
     try:
         os.makedirs(dir)
@@ -43,10 +49,21 @@ def _get_outstream(outfile: str) -> Any:
     return open(outfile, mode="w", encoding="utf-8")
 
 
+def syntax_check(filename: str) -> bool:
+    with open(filename) as f:
+        source = f.read()
+    valid = True
+    try:
+        ast.parse(source)
+    except SyntaxError:
+        valid = False
+    return valid
+
+
 def decompile(
-    bytecode_version: str,
     co,
-    out=None,
+    bytecode_version: Tuple[int] = PYTHON_VERSION_TRIPLE,
+    out=sys.stdout,
     showasm=None,
     showast={},
     timestamp=None,
@@ -87,7 +104,7 @@ def decompile(
         write("# -*- coding: %s -*-" % source_encoding)
     write(
         "# decompyle3 version %s\n"
-        "# %sPython bytecode %s%s\n# Decompiled from: %sPython %s"
+        "# %sPython bytecode version base %s%s\n# Decompiled from: %sPython %s"
         % (
             __version__,
             co_pypy_str,
@@ -104,7 +121,13 @@ def decompile(
     if source_size:
         write("# Size of source mod 2**32: %d bytes" % source_size)
 
-    debug_opts = {"asm": showasm, "ast": showast, "grammar": showgrammar}
+    # maybe a second -a will do before as well
+    asm = "after" if showasm else None
+
+    grammar = dict(PARSER_DEFAULT_DEBUG)
+    if showgrammar:
+        grammar["reduce"] = True
+    debug_opts = {"asm": asm, "tree": showast, "grammar": grammar}
 
     try:
         if mapstream:
@@ -186,11 +209,11 @@ def decompile_file(
 
     if isinstance(co, list):
         deparsed = []
-        for con in co:
+        for bytecode in co:
             deparsed.append(
                 decompile(
+                    bytecode,
                     version,
-                    con,
                     outstream,
                     showasm,
                     showast,
@@ -206,8 +229,8 @@ def decompile_file(
     else:
         deparsed = [
             decompile(
-                version,
                 co,
+                version,
                 outstream,
                 showasm,
                 showast,
@@ -236,7 +259,7 @@ def main(
     outfile=None,
     showasm=None,
     showast={},
-    do_verify=False,
+    do_verify=Optional[str],
     showgrammar=False,
     source_encoding=None,
     raise_on_error=False,
@@ -254,7 +277,8 @@ def main(
     - files below out_base	out_base=...
     - stdout			out_base=None, outfile=None
     """
-    tot_files = okay_files = failed_files = verify_failed_files = 0
+    tot_files = okay_files = failed_files = skipped = 0
+    verify_failed_files = 0 if do_verify is not None else None
     current_outfile = outfile
     linemap_stream = None
 
@@ -262,10 +286,11 @@ def main(
         compiled_files.append(compile_file(source_path))
 
     for filename in compiled_files:
-        infile = os.path.join(in_base, filename)
+        infile = osp.join(in_base, filename)
         # print("XXX", infile)
-        if not os.path.exists(infile):
+        if not osp.exists(infile):
             sys.stderr.write("File '%s' doesn't exist. Skipped\n" % infile)
+            skipped += 1
             continue
 
         if do_linemaps:
@@ -277,14 +302,19 @@ def main(
         if outfile:  # outfile was given as parameter
             outstream = _get_outstream(outfile)
         elif out_base is None:
-            outstream = sys.stdout
+            out_base = tempfile.mkdtemp(prefix="py-dis-")
+            if do_verify and filename.endswith(".pyc"):
+                current_outfile = osp.join(out_base, filename[0:-1])
+                outstream = open(current_outfile, "w")
+            else:
+                outstream = sys.stdout
             if do_linemaps:
                 linemap_stream = sys.stdout
         else:
             if filename.endswith(".pyc"):
-                current_outfile = os.path.join(out_base, filename[0:-1])
+                current_outfile = osp.join(out_base, filename[0:-1])
             else:
-                current_outfile = os.path.join(out_base, filename) + "_dis"
+                current_outfile = osp.join(out_base, filename) + "_dis"
                 pass
             pass
 
@@ -294,7 +324,7 @@ def main(
 
         # Try to uncompile the input file
         try:
-            deparsed = decompile_file(
+            deparsed_objects = decompile_file(
                 infile,
                 outstream,
                 showasm,
@@ -305,9 +335,9 @@ def main(
                 do_fragments,
             )
             if do_fragments:
-                for d in deparsed:
+                for deparsed_object in deparsed_objects:
                     last_mod = None
-                    offsets = d.offsets
+                    offsets = deparsed_object.offsets
                     for e in sorted(
                         [k for k in offsets.keys() if isinstance(k[1], int)]
                     ):
@@ -316,12 +346,47 @@ def main(
                             outstream.write("%s\n%s\n%s\n" % (line, e[0], line))
                         last_mod = e[0]
                         info = offsets[e]
-                        extractInfo = d.extract_node_info(info)
+                        extractInfo = deparsed_object.extract_node_info(info)
                         outstream.write("%s" % info.node.format().strip() + "\n")
                         outstream.write(extractInfo.selectedLine + "\n")
                         outstream.write(extractInfo.markerLine + "\n\n")
                     pass
                 pass
+            if do_verify:
+                for deparsed_object in deparsed_objects:
+                    deparsed_object.f.close()
+                    if PYTHON_VERSION_TRIPLE[:2] != deparsed_object.version[:2]:
+                        sys.stdout.write(
+                            f"\n# skipping running {deparsed_object.f.name} "
+                            f"it is {version_tuple_to_str(deparsed_object.version, end=2)}, "
+                            f"and we are {version_tuple_to_str(PYTHON_VERSION_TRIPLE, end=2)}\n"
+                        )
+                    else:
+                        check_type = "syntax check"
+                        if do_verify == "run":
+                            check_type = "run"
+                            result = subprocess.run(
+                                [sys.executable, deparsed_object.f.name],
+                                capture_output=True,
+                            )
+                            valid = result.returncode == 0
+                            output = result.stdout.decode()
+                            if output:
+                                print(output)
+                            pass
+                            if not valid:
+                                print(result.stderr.decode())
+
+                        else:
+                            valid = syntax_check(deparsed_object.f.name)
+
+                        if not valid:
+                            verify_failed_files += 1
+                            sys.stderr.write(
+                                f"\n# {check_type} failed on file {deparsed_object.f.name}\n"
+                            )
+
+                    # sys.stderr.write(f"Ran {deparsed_object.f.name}\n")
             tot_files += 1
         except (ValueError, SyntaxError, ParserError, pysource.SourceWalkerError) as e:
             sys.stdout.write("\n")
@@ -380,21 +445,20 @@ def main(
                         okay_files,
                         failed_files,
                         verify_failed_files,
-                        do_verify,
                     ),
                 )
             )
             try:
                 # FIXME: Something is weird with Pypy here
                 sys.stdout.flush()
-            except:
+            except Exception:
                 pass
     if current_outfile:
         sys.stdout.write("\n")
         try:
             # FIXME: Something is weird with Pypy here
             sys.stdout.flush()
-        except:
+        except Exception:
             pass
         pass
     return (tot_files, okay_files, failed_files, verify_failed_files)
@@ -418,22 +482,24 @@ else:
 
 
 def status_msg(
-    do_verify, tot_files, okay_files, failed_files, verify_failed_files, weak_verify
+    do_verify: bool,
+    tot_files: int,
+    okay_files: int,
+    failed_files: int,
+    verify_failed_files: Optional[int],
 ):
-    if weak_verify == "weak":
-        verification_type = "weak "
-    elif weak_verify == "verify-run":
-        verification_type = "run "
-    else:
-        verification_type = ""
     if tot_files == 1:
         if failed_files:
             return "\n# decompile failed"
         elif verify_failed_files:
-            return f"\n# decompile {verification_type}verification failed"
+            return "\n# decompile run verification failed"
+        elif do_verify:
+            return "\n# Successfully decompiled and ran or syntax-checked all files"
         else:
-            return "\n# Successfully decompiled file"
+            return "\n# Successfully decompiled all files"
             pass
         pass
     mess = f"decompiled {tot_files} files: {okay_files} okay, {failed_files} failed"
+    if do_verify:
+        mess += f", {verify_failed_files} failed verification"
     return mess
