@@ -1,13 +1,17 @@
-#  Copyright (c) 2016-2017, 2019-2021 Rocky Bernstein
+#  Copyright (c) 2016-2017, 2019-2022 Rocky Bernstein
 """
 Python 3.7 base code. We keep non-custom-generated grammar rules out of this file.
 """
-from decompyle3.parsers.main import PythonParser, nop_func, ParserError
+from decompyle3.parsers.parse_heads import (
+    ParserError,
+    PythonBaseParser,
+    nop_func,
+)
 from decompyle3.parsers.treenode import SyntaxTree
 from spark_parser import DEFAULT_DEBUG as PARSER_DEFAULT_DEBUG
 from spark_parser.spark import rule2str
 
-from decompyle3.parsers.reducecheck import (
+from decompyle3.parsers.reduce_check import (
     and_check,
     and_cond_check,
     and_not_check,
@@ -31,13 +35,11 @@ from decompyle3.parsers.reducecheck import (
 )
 
 
-class Python37BaseParser(PythonParser):
-    def __init__(self, debug_parser=PARSER_DEFAULT_DEBUG, compile_mode="exec"):
+class Python37BaseParser(PythonBaseParser):
+    def __init__(self, debug_parser=PARSER_DEFAULT_DEBUG):
 
         self.added_rules = set()
-        super(Python37BaseParser, self).__init__(
-            SyntaxTree, compile_mode, debug=debug_parser
-        )
+        super(Python37BaseParser, self).__init__(SyntaxTree, debug=debug_parser)
         self.new_rules = set()
 
     @staticmethod
@@ -65,7 +67,7 @@ class Python37BaseParser(PythonParser):
                         expr
                         call
                         CALL_FUNCTION_3
-         """
+        """
         # FIXME: I bet this can be simplified
         # look for next MAKE_FUNCTION
         for i in range(i + 1, len(tokens)):
@@ -114,8 +116,7 @@ class Python37BaseParser(PythonParser):
     # FIXME FIXME FIXME: The below is an utter mess. Come up with a better
     # organization for this. For example, arrange organize by opcode base?
 
-    def customize_grammar_rules(self, tokens, customize):
-
+    def customize_grammar_rules37(self, tokens, customize):
         is_pypy = False
 
         # For a rough break out on the first word. This may
@@ -293,17 +294,7 @@ class Python37BaseParser(PythonParser):
                                                COME_FROM_ASYNC_WITH
                                                WITH_CLEANUP_START GET_AWAITABLE LOAD_CONST YIELD_FROM
                                                WITH_CLEANUP_FINISH END_FINALLY
-                      c_async_with_stmt   ::= expr
-                                              async_with_pre
-                                              POP_TOP
-                                              c_suite_stmts
-                                              POP_BLOCK
-                                              BEGIN_FINALLY
-                                              WITH_CLEANUP_START GET_AWAITABLE LOAD_CONST YIELD_FROM
-                                              WITH_CLEANUP_FINISH POP_FINALLY LOAD_CONST RETURN_VALUE
-                                              COME_FROM_ASYNC_WITH
-                                              WITH_CLEANUP_START GET_AWAITABLE LOAD_CONST YIELD_FROM
-                                              WITH_CLEANUP_FINISH END_FINALLY
+                      c_async_with_stmt   ::= async_with_stmt
                       async_with_as_stmt   ::= expr
                                                async_with_pre
                                                store suite_stmts
@@ -348,8 +339,8 @@ class Python37BaseParser(PythonParser):
                 if opname == "BUILD_MAP_UNPACK":
                     self.addRule(
                         """
-                        expr       ::= unmap_dict
-                        unmap_dict ::= expr BUILD_MAP_UNPACK
+                        expr        ::= dict_unpack
+                        dict_unpack ::= expr BUILD_MAP_UNPACK
                         """,
                         nop_func,
                     )
@@ -363,8 +354,8 @@ class Python37BaseParser(PythonParser):
                 if opname == "BUILD_MAP_n":
                     # PyPy sometimes has no count. Sigh.
                     rule = (
-                        "dict_comp_func ::= BUILD_MAP_n LOAD_FAST for_iter store "
-                        "comp_iter JUMP_BACK RETURN_VALUE RETURN_LAST"
+                        "dict_comp_func ::= BUILD_MAP_n LOAD_ARG for_iter store "
+                        "comp_iter JUMP_LOOP RETURN_VALUE RETURN_LAST"
                     )
                     self.add_unique_rule(rule, "dict_comp_func", 1, customize)
 
@@ -382,7 +373,7 @@ class Python37BaseParser(PythonParser):
                     # FIXME: Use the attr
                     # so this doesn't run into exponential parsing time.
                     if opname.startswith("BUILD_MAP_UNPACK"):
-                        # FIXME: start here. The LHS should be unmap_dict, not dict.
+                        # FIXME: start here. The LHS should be dict_unpack, not dict.
                         # FIXME: really we need a combination of dict_entry-like things.
                         # It just so happens the most common case is not to mix
                         # dictionary comphensions with dictionary, elements
@@ -397,8 +388,8 @@ class Python37BaseParser(PythonParser):
                             )
                             self.addRule(rule, nop_func)
                         rule = """
-                         expr       ::= unmap_dict
-                         unmap_dict ::= %s%s
+                         expr        ::= dict_unpack
+                         dict_unpack ::= %s%s
                          """ % (
                             "expr " * token.attr,
                             opname,
@@ -443,35 +434,39 @@ class Python37BaseParser(PythonParser):
                 "BUILD_TUPLE",
                 "BUILD_TUPLE_UNPACK",
             ):
-                v = token.attr
+                collection_size = token.attr
 
                 is_LOAD_CLOSURE = False
                 if opname_base == "BUILD_TUPLE":
                     # If is part of a "load_closure", then it is not part of a
                     # "list".
                     is_LOAD_CLOSURE = True
-                    for j in range(v):
+                    for j in range(collection_size):
                         if tokens[i - j - 1].kind != "LOAD_CLOSURE":
                             is_LOAD_CLOSURE = False
                             break
                     if is_LOAD_CLOSURE:
-                        rule = "load_closure ::= %s%s" % (("LOAD_CLOSURE " * v), opname)
+                        rule = "load_closure ::= %s%s" % (
+                            ("LOAD_CLOSURE " * collection_size),
+                            opname,
+                        )
                         self.add_unique_rule(rule, opname, token.attr, customize)
-                if not is_LOAD_CLOSURE or v == 0:
+                if not is_LOAD_CLOSURE or collection_size == 0:
                     # We do this complicated test to speed up parsing of
                     # pathelogically long literals, especially those over 1024.
-                    build_count = token.attr
-                    thousands = build_count // 1024
-                    thirty32s = (build_count // 32) % 32
+                    thousands = collection_size // 1024
+                    thirty32s = (collection_size // 32) % 32
                     if thirty32s > 0:
                         rule = "expr32 ::=%s" % (" expr" * 32)
-                        self.add_unique_rule(rule, opname_base, build_count, customize)
+                        self.add_unique_rule(
+                            rule, opname_base, collection_size, customize
+                        )
                         pass
                     if thousands > 0:
                         self.add_unique_rule(
                             "expr1024 ::=%s" % (" expr32" * 32),
                             opname_base,
-                            build_count,
+                            collection_size,
                             customize,
                         )
                         pass
@@ -480,7 +475,7 @@ class Python37BaseParser(PythonParser):
                         ("%s ::= " % collection)
                         + "expr1024 " * thousands
                         + "expr32 " * thirty32s
-                        + "expr " * (build_count % 32)
+                        + "expr " * (collection_size % 32)
                         + opname
                     )
                     self.add_unique_rules(["expr ::= %s" % collection, rule], customize)
@@ -489,20 +484,17 @@ class Python37BaseParser(PythonParser):
             elif opname_base == "BUILD_SLICE":
                 if token.attr == 2:
                     self.add_unique_rules(
-                        [
-                            "expr ::= build_slice2",
-                            "build_slice2 ::= expr expr BUILD_SLICE_2",
-                        ],
+                        ["expr ::= slice2", "slice2 ::= expr expr BUILD_SLICE_2",],
                         customize,
                     )
                 else:
                     assert token.attr == 3, (
-                        "BUILD_SLICE value must be 2 or 3; is %s" % v
+                        "BUILD_SLICE value must be 2 or 3; is %s" % token.attr
                     )
                     self.add_unique_rules(
                         [
-                            "expr ::= build_slice3",
-                            "build_slice3 ::= expr expr expr BUILD_SLICE_3",
+                            "expr   ::= slice3",
+                            "slice3 ::= expr expr expr BUILD_SLICE_3",
                         ],
                         customize,
                     )
@@ -550,17 +542,31 @@ class Python37BaseParser(PythonParser):
             elif opname_base == "CALL_METHOD":
                 # PyPy and Python 3.7+ only - DRY with parse2
 
-                args_pos, args_kw = self.get_pos_kw(token)
+                if opname == "CALL_METHOD_KW":
+                    args_kw = token.attr
+                    rules_str = """
+                         expr ::= call_kw_pypy37
+                         pypy_kw_keys ::= LOAD_CONST
+                    """
+                    self.add_unique_doc_rules(rules_str, customize)
+                    rule = (
+                        "call_kw_pypy37 ::= expr "
+                        + ("expr " * args_kw)
+                        + " pypy_kw_keys "
+                        + opname
+                    )
+                else:
+                    args_pos, args_kw = self.get_pos_kw(token)
+                    # number of apply equiv arguments:
+                    nak = (len(opname_base) - len("CALL_METHOD")) // 3
+                    rule = (
+                        "call ::= expr "
+                        + ("expr " * args_pos)
+                        + ("kwarg " * args_kw)
+                        + "expr " * nak
+                        + opname
+                    )
 
-                # number of apply equiv arguments:
-                nak = (len(opname_base) - len("CALL_METHOD")) // 3
-                rule = (
-                    "call ::= expr "
-                    + ("expr " * args_pos)
-                    + ("kwarg " * args_kw)
-                    + "expr " * nak
-                    + opname
-                )
                 self.add_unique_rule(rule, opname, token.attr, customize)
 
             elif opname == "CONTINUE":
@@ -605,48 +611,10 @@ class Python37BaseParser(PythonParser):
                 """
                 self.add_unique_doc_rules(rules_str, customize)
 
-            elif opname == "GET_ITER":
-                self.addRule(
-                    """
-                    expr      ::= get_iter
-                    get_iter  ::= expr GET_ITER
-                    """,
-                    nop_func,
-                )
-                custom_ops_processed.add(opname)
             elif opname == "GET_AITER":
                 self.addRule(
                     """
-                    expr                ::= generator_exp_async
-                    generator_exp_async ::= load_genexpr LOAD_STR MAKE_FUNCTION_0 expr
-                                            GET_AITER CALL_FUNCTION_1
-
                     stmt                ::= genexpr_func_async
-
-                    func_async_prefix   ::= _come_froms SETUP_EXCEPT GET_ANEXT LOAD_CONST YIELD_FROM
-                    func_async_middle   ::= POP_BLOCK JUMP_FORWARD COME_FROM_EXCEPT
-                                            DUP_TOP LOAD_GLOBAL COMPARE_OP POP_JUMP_IF_TRUE
-                                            END_FINALLY COME_FROM
-                    genexpr_func_async  ::= LOAD_FAST func_async_prefix
-                                            store func_async_middle comp_iter
-                                            JUMP_BACK COME_FROM
-                                            POP_TOP POP_TOP POP_TOP POP_EXCEPT POP_TOP
-
-                    expr                ::= list_comp_async
-                    list_comp_async     ::= LOAD_LISTCOMP LOAD_STR MAKE_FUNCTION_0
-                                            expr GET_AITER CALL_FUNCTION_1
-                                            GET_AWAITABLE LOAD_CONST
-                                            YIELD_FROM
-
-                    expr                ::= list_comp_async
-                    list_afor2          ::= func_async_prefix
-                                            store func_async_middle list_iter
-                                            JUMP_BACK COME_FROM
-                                            POP_TOP POP_TOP POP_TOP POP_EXCEPT POP_TOP
-                    list_comp_async     ::= BUILD_LIST_0 LOAD_FAST list_afor2
-                    get_aiter           ::= expr GET_AITER
-                    list_afor           ::= get_aiter list_afor2
-                    list_iter           ::= list_afor
                    """,
                     nop_func,
                 )
@@ -688,15 +656,6 @@ class Python37BaseParser(PythonParser):
                     self.add_make_function_rule(rule_pat, opname, token.attr, customize)
                     pass
                 custom_ops_processed.add(opname)
-            elif opname == "LOAD_ATTR":
-                self.addRule(
-                    """
-                  expr      ::= attribute
-                  attribute ::= expr LOAD_ATTR
-                  """,
-                    nop_func,
-                )
-                custom_ops_processed.add(opname)
             elif opname == "LOAD_LISTCOMP":
                 self.add_unique_rule(
                     "expr ::= list_comp", opname, token.attr, customize
@@ -759,8 +718,8 @@ class Python37BaseParser(PythonParser):
                 j = 2
                 if is_pypy or (i >= j and tokens[i - j] == "LOAD_LAMBDA"):
                     rule_pat = """
-                                expr     ::= mklambda
-                                mklambda ::= %sload_closure LOAD_LAMBDA %%s%s
+                                expr        ::= lambda_body
+                                lambda_body ::= %sload_closure LOAD_LAMBDA %%s%s
                                """ % (
                         "expr " * args_pos,
                         opname,
@@ -836,8 +795,8 @@ class Python37BaseParser(PythonParser):
                 if closure:
                     if args_pos:
                         rule = """
-                             expr     ::= mklambda
-                             mklambda ::= %s%s%s%s
+                             expr        ::= lambda_body
+                             lambda_body ::= %s%s%s%s
                              """ % (
                             "expr " * stack_count,
                             "load_closure " * closure,
@@ -846,8 +805,8 @@ class Python37BaseParser(PythonParser):
                         )
                     else:
                         rule = """
-                             expr     ::= mklambda
-                             mklambda ::= %s%s%s""" % (
+                             expr        ::= lambda_body
+                             lambda_body ::= %s%s%s""" % (
                             "load_closure " * closure,
                             "LOAD_LAMBDA LOAD_STR ",
                             opname,
@@ -856,8 +815,8 @@ class Python37BaseParser(PythonParser):
 
                 else:
                     rule = """
-                         expr     ::= mklambda
-                         mklambda ::= %sLOAD_LAMBDA LOAD_STR %s""" % (
+                         expr        ::= lambda_body
+                         lambda_body ::= %sLOAD_LAMBDA LOAD_STR %s""" % (
                         ("expr " * stack_count),
                         opname,
                     )
@@ -874,8 +833,8 @@ class Python37BaseParser(PythonParser):
                 if has_get_iter_call_function1:
                     rule_pat = (
                         "generator_exp ::= %sload_genexpr %%s%s expr "
-                        "GET_ITER CALL_FUNCTION_1" % ("expr " * args_pos, opname)
-                    )
+                        "GET_ITER CALL_FUNCTION_1"
+                    ) % ("expr " * args_pos, opname)
                     self.add_make_function_rule(rule_pat, opname, token.attr, customize)
                     rule_pat = """
                            expr          ::= generator_exp
@@ -906,8 +865,8 @@ class Python37BaseParser(PythonParser):
 
                 if is_pypy or (i >= 2 and tokens[i - 2] == "LOAD_LAMBDA"):
                     rule_pat = """
-                        expr     ::= mklambda
-                        mklambda ::= %s%sLOAD_LAMBDA %%s%s
+                        expr        ::= lambda_body
+                        lambda_body ::= %s%sLOAD_LAMBDA %%s%s
                         """ % (
                         ("expr " * args_pos),
                         ("kwarg " * args_kw),
@@ -944,8 +903,8 @@ class Python37BaseParser(PythonParser):
                 # FIXME: Fold test  into add_make_function_rule
                 if is_pypy or (i >= j and tokens[i - j] == "LOAD_LAMBDA"):
                     rule_pat = """
-                        expr     ::= mklambda
-                        mklambda ::= %s%sLOAD_LAMBDA %%s%s
+                        expr        ::= lambda_body
+                        lambda_body ::= %s%sLOAD_LAMBDA %%s%s
                         """ % (
                         ("expr " * args_pos),
                         ("kwarg " * args_kw),
@@ -968,29 +927,10 @@ class Python37BaseParser(PythonParser):
                 )
                 self.add_unique_rule(rule, opname, token.attr, customize)
 
-            elif opname == "MAKE_FUNCTION_8":
-                if "LOAD_DICTCOMP" in self.seen_ops:
-                    # Is there something general going on here?
-                    rule = """
-                       expr      ::= dict_comp
-                       dict_comp ::= load_closure LOAD_DICTCOMP LOAD_STR
-                                     MAKE_FUNCTION_8 expr
-                                     GET_ITER CALL_FUNCTION_1
-                       """
-                    self.addRule(rule, nop_func)
-                elif "LOAD_SETCOMP" in self.seen_ops:
-                    self.addRule("expr ::= set_comp", nop_func)
-                    rule = """
-                       set_comp ::= load_closure LOAD_SETCOMP LOAD_STR
-                                    MAKE_FUNCTION_8 expr
-                                    GET_ITER CALL_FUNCTION_1
-                       """
-                    self.addRule(rule, nop_func)
-
             elif opname == "RETURN_VALUE_LAMBDA":
                 self.addRule(
                     """
-                    return_lambda ::= ret_expr RETURN_VALUE_LAMBDA
+                    return_expr_lambda ::= return_expr RETURN_VALUE_LAMBDA
                     """,
                     nop_func,
                 )
@@ -1112,7 +1052,7 @@ class Python37BaseParser(PythonParser):
                     rules_str += """
                      # A return at the end of a withas stmt can be this.
                      # FIXME: should this be a different kind of return?
-                     return      ::= ret_expr POP_BLOCK
+                     return      ::= return_expr POP_BLOCK
                                      ROT_TWO
                                      BEGIN_FINALLY
                                      WITH_CLEANUP_START
@@ -1214,7 +1154,9 @@ class Python37BaseParser(PythonParser):
         self.check_reduce["ifstmtc"] = "AST"
         self.check_reduce["ifstmts_jump"] = "AST"
         self.check_reduce["ifstmts_jumpc"] = "AST"
+        self.check_reduce["import_as37"] = "tokens"
         self.check_reduce["import_from37"] = "AST"
+        self.check_reduce["import_from_as37"] = "tokens"
         self.check_reduce["lastc_stmt"] = "tokens"
         self.check_reduce["list_if_not"] = "AST"
         self.check_reduce["while1elsestmt"] = "tokens"
@@ -1314,7 +1256,7 @@ class Python37BaseParser(PythonParser):
         try:
             if fn:
                 return fn(self, lhs, n, rule, ast, tokens, first, last)
-        except:
+        except Exception:
             import sys, traceback
 
             print(
@@ -1337,5 +1279,9 @@ class Python37BaseParser(PythonParser):
                 assert store == "store"
                 return alias37[0].attr != store[0].attr
             return False
+        elif lhs == "import_as37":
+            return tokens[first + 1].pattr is not None
+        elif lhs == "import_from_as37":
+            return tokens[first + 1].pattr is None
 
         return False
