@@ -1,4 +1,4 @@
-#  Copyright (c) 2017-2020 Rocky Bernstein
+#  Copyright (c) 2017-2022 Rocky Bernstein
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -16,18 +16,30 @@
 Python 3.7 lambda grammar for the spark Earley-algorithm parser.
 """
 
-from decompyle3.parsers.main import nop_func
+from decompyle3.parsers.p37.lambda_custom import Python37LambdaCustom
+from decompyle3.parsers.parse_heads import (
+    PythonParserLambda,
+    PythonBaseParser,
+)
 from spark_parser import DEFAULT_DEBUG as PARSER_DEFAULT_DEBUG
-from decompyle3.parsers.p37.base import Python37BaseParser
-from decompyle3.scanners.tok import Token
 
 
-class Python37LambdaParser(Python37BaseParser):
-    def __init__(self, debug_parser=PARSER_DEFAULT_DEBUG, compile_mode="lambda"):
-        super(Python37LambdaParser, self).__init__(
-            debug_parser, compile_mode=compile_mode
+class Python37LambdaParser(Python37LambdaCustom, PythonParserLambda):
+    def __init__(
+        self,
+        start_symbol: str = "lambda_start",
+        debug_parser: dict = PARSER_DEFAULT_DEBUG,
+    ):
+        PythonParserLambda.__init__(
+            self, debug_parser=debug_parser, start_symbol=start_symbol
         )
-        self.customized = {}
+        PythonBaseParser.__init__(
+            self, start_symbol=start_symbol, debug_parser=debug_parser
+        )
+        Python37LambdaCustom.__init__(self)
+
+    def customize_grammar_rules(self, tokens, customize):
+        self.customize_grammar_rules_lambda37(tokens, customize)
 
     ###################################################
     #  Python 3.7 grammar rules for lambda expressions
@@ -36,23 +48,34 @@ class Python37LambdaParser(Python37BaseParser):
 
     def p_lambda(self, args):
         """
-        lambda_start       ::= return_lambda LAMBDA_MARKER
-        return_lambda      ::= expr RETURN_VALUE_LAMBDA
-        return_lambda      ::= if_exp_lambda
-        return_lambda      ::= if_exp_lambda2
-        return_lambda      ::= if_exp_not_lambda
-        return_lambda      ::= if_exp_dead_code
+        lambda_start       ::= return_expr_lambda LAMBDA_MARKER
+        return_expr_lambda ::= expr RETURN_VALUE_LAMBDA
+        return_expr_lambda ::= if_exp_lambda
+        return_expr_lambda ::= if_exp_lambda2
+        return_expr_lambda ::= if_exp_not_lambda
+        return_expr_lambda ::= if_exp_not_lambda2
+        return_expr_lambda ::= if_exp_dead_code
+        return_expr_lambda ::= dict_comp_func
+
+        ## FIXME: add rules for these
+        # return_expr_lambda ::= generator_exp
+        # return_expr_lambda ::= list_comp_func
+
+        return_expr_lambda ::= set_comp_func
+
 
         return_if_lambda   ::= RETURN_END_IF_LAMBDA COME_FROM
         return_if_lambda   ::= RETURN_END_IF_LAMBDA
 
         if_exp_lambda      ::= expr_pjif expr return_if_lambda
-                               return_lambda LAMBDA_MARKER
-        if_exp_lambda2     ::= and_parts return_lambda come_froms
-                               return_lambda opt_lambda_marker
+                               return_expr_lambda LAMBDA_MARKER
+        if_exp_lambda2     ::= and_parts return_expr_lambda come_froms
+                               return_expr_lambda opt_lambda_marker
         if_exp_not_lambda  ::= expr POP_JUMP_IF_TRUE expr return_if_lambda
-                               return_lambda LAMBDA_MARKER
-        if_exp_dead_code   ::= return_lambda return_lambda
+                               return_expr_lambda LAMBDA_MARKER
+        if_exp_not_lambda2 ::= expr POP_JUMP_IF_TRUE expr
+                               RETURN_VALUE_LAMBDA COME_FROM return_expr_lambda
+        if_exp_dead_code   ::= return_expr_lambda return_expr_lambda
         opt_lambda_marker  ::= LAMBDA_MARKER?
         """
 
@@ -139,9 +162,6 @@ class Python37LambdaParser(Python37BaseParser):
         #     i and (j or k)
 
         or        ::= expr_jitop expr come_from_opt
-
-
-        or        ::= expr_pjit expr COME_FROM
         or_expr   ::= expr JUMP_IF_TRUE expr COME_FROM
 
         jitop_come_from_expr ::= JUMP_IF_TRUE_OR_POP _come_froms expr
@@ -165,7 +185,7 @@ class Python37LambdaParser(Python37BaseParser):
     def p_jump(self, args):
         """
         jump               ::= JUMP_FORWARD
-        jump               ::= JUMP_BACK
+        jump               ::= JUMP_LOOP
         jump_or_break      ::= jump
         jump_or_break      ::= BREAK_LOOP
 
@@ -175,10 +195,10 @@ class Python37LambdaParser(Python37BaseParser):
         jump_forward_else  ::= come_froms jump COME_FROM
 
         pjump_ift          ::= POP_JUMP_IF_TRUE
-        pjump_ift          ::= POP_JUMP_IF_TRUE_BACK
+        pjump_ift          ::= POP_JUMP_IF_TRUE_LOOP
 
         pjump_iff          ::= POP_JUMP_IF_FALSE
-        pjump_iff          ::= POP_JUMP_IF_FALSE_BACK
+        pjump_iff          ::= POP_JUMP_IF_FALSE_LOOP
 
         # pjump              ::= pjump_iff
         # pjump              ::= pjump_ift
@@ -228,7 +248,7 @@ class Python37LambdaParser(Python37BaseParser):
         chained_part               ::= expr DUP_TOP ROT_THREE COMPARE_OP come_from_opt POP_JUMP_IF_FALSE
 
         # c_chained_parts            ::= c_chained_part+
-        # c_chained_part             ::= expr DUP_TOP ROT_THREE COMPARE_OP come_from_opt POP_JUMP_IF_FALSE_BACK
+        # c_chained_part             ::= expr DUP_TOP ROT_THREE COMPARE_OP come_from_opt POP_JUMP_IF_FALSE_LOOP
         # c_chained_parts            ::= chained_parts
 
 
@@ -256,31 +276,30 @@ class Python37LambdaParser(Python37BaseParser):
         compare_chained1_false_37  ::= chained_parts
                                        compare_chained2b_false_37 POP_TOP jump COME_FROM
 
-        compare_chained2           ::= expr COMPARE_OP JUMP_FORWARD
         compare_chained2           ::= expr COMPARE_OP JUMP_FORWARD
         compare_chained2           ::= expr COMPARE_OP RETURN_VALUE
         compare_chained2           ::= expr COMPARE_OP RETURN_VALUE_LAMBDA
 
         compare_chained2_false_37  ::= chained_parts
-                                      compare_chained2a_false_37 POP_TOP JUMP_BACK COME_FROM
+                                      compare_chained2a_false_37 POP_TOP JUMP_LOOP COME_FROM
         c_compare_chained2_false_37  ::= chained_parts
-                                         c_compare_chained2a_false_37 POP_TOP JUMP_BACK COME_FROM
+                                         c_compare_chained2a_false_37 POP_TOP JUMP_LOOP COME_FROM
 
         compare_chained2a_37       ::= expr COMPARE_OP come_from_opt POP_JUMP_IF_TRUE JUMP_FORWARD
-        c_compare_chained2a_37     ::= expr COMPARE_OP come_from_opt POP_JUMP_IF_TRUE_BACK JUMP_FORWARD
+        c_compare_chained2a_37     ::= expr COMPARE_OP come_from_opt POP_JUMP_IF_TRUE_LOOP JUMP_FORWARD
 
 
-        compare_chained2a_37       ::= expr COMPARE_OP come_from_opt POP_JUMP_IF_TRUE JUMP_BACK
+        compare_chained2a_37       ::= expr COMPARE_OP come_from_opt POP_JUMP_IF_TRUE JUMP_LOOP
         compare_chained2a_false_37 ::= expr COMPARE_OP come_from_opt POP_JUMP_IF_FALSE jf_cfs
 
 
         compare_chained2b_false_37   ::= expr COMPARE_OP come_from_opt POP_JUMP_IF_FALSE
                                          jump_or_break COME_FROM
-        c_compare_chained2b_false_37 ::= expr COMPARE_OP come_from_opt POP_JUMP_IF_FALSE_BACK
+        c_compare_chained2b_false_37 ::= expr COMPARE_OP come_from_opt POP_JUMP_IF_FALSE_LOOP
                                          jump_or_break COME_FROM
-        c_compare_chained2a_false_37 ::= expr COMPARE_OP come_from_opt POP_JUMP_IF_FALSE_BACK
+        c_compare_chained2a_false_37 ::= expr COMPARE_OP come_from_opt POP_JUMP_IF_FALSE_LOOP
                                          jf_cfs
-        c_compare_chained2a_false_37 ::= expr COMPARE_OP come_from_opt POP_JUMP_IF_FALSE_BACK
+        c_compare_chained2a_false_37 ::= expr COMPARE_OP come_from_opt POP_JUMP_IF_FALSE_LOOP
         c_compare_chained2b_false_37 ::= expr COMPARE_OP come_from_opt JUMP_FORWARD COME_FROM
 
 
@@ -296,7 +315,9 @@ class Python37LambdaParser(Python37BaseParser):
         expr ::= LOAD_GLOBAL
         expr ::= LOAD_NAME
         expr ::= LOAD_STR
+
         expr ::= and
+        expr ::= attribute37
         expr ::= or_and
         expr ::= and_or
         expr ::= and_or_expr
@@ -305,13 +326,18 @@ class Python37LambdaParser(Python37BaseParser):
         expr ::= compare
         expr ::= or
         expr ::= or_expr
+        expr ::= if_exp
+        expr ::= if_exp_loop
+
+        expr ::= list_comp
+
+        expr ::= set_comp
         expr ::= subscript
         expr ::= subscript2
         expr ::= unary_not
         expr ::= unary_op
         expr ::= not
         expr ::= yield
-        expr ::= attribute37
 
         # Python 3.3+ adds yield from.
         expr          ::= yield_from
@@ -323,17 +349,36 @@ class Python37LambdaParser(Python37BaseParser):
         bin_op            ::= expr expr binary_operator
 
         binary_operator   ::= BINARY_ADD
-        binary_operator   ::= BINARY_MULTIPLY
         binary_operator   ::= BINARY_AND
+        binary_operator   ::= BINARY_FLOOR_DIVIDE
+        binary_operator   ::= BINARY_LSHIFT
+        binary_operator   ::= BINARY_MATRIX_MULTIPLY
+        binary_operator   ::= BINARY_MODULO
+        binary_operator   ::= BINARY_MULTIPLY
         binary_operator   ::= BINARY_OR
-        binary_operator   ::= BINARY_XOR
+        binary_operator   ::= BINARY_POWER
+        binary_operator   ::= BINARY_RSHIFT
         binary_operator   ::= BINARY_SUBTRACT
         binary_operator   ::= BINARY_TRUE_DIVIDE
-        binary_operator   ::= BINARY_FLOOR_DIVIDE
-        binary_operator   ::= BINARY_MODULO
-        binary_operator   ::= BINARY_LSHIFT
-        binary_operator   ::= BINARY_RSHIFT
-        binary_operator   ::= BINARY_POWER
+        binary_operator   ::= BINARY_XOR
+
+        # FIXME: the below is to work around test_grammar expecting a "call" to be
+        # on the LHS because it is also somewhere on in a rule.
+        call              ::= expr CALL_METHOD_0
+
+        compare           ::= compare_chained
+        compare           ::= compare_single
+        compare_single    ::= expr expr COMPARE_OP
+        c_compare         ::= c_compare_chained
+
+        genexpr_func      ::= LOAD_ARG _come_froms FOR_ITER store comp_iter
+                              JUMP_LOOP _come_froms
+
+        load_genexpr      ::= LOAD_GENEXPR
+        load_genexpr      ::= BUILD_TUPLE_1 LOAD_GENEXPR LOAD_STR
+
+        subscript         ::= expr expr BINARY_SUBSCR
+        subscript2        ::= expr expr DUP_TOP_TWO BINARY_SUBSCR
 
         # unary_op (formerly "unary_expr") is the Python AST UnaryOp
         unary_op          ::= expr unary_operator
@@ -344,44 +389,48 @@ class Python37LambdaParser(Python37BaseParser):
 
         unary_not         ::= expr UNARY_NOT
 
-        subscript         ::= expr expr BINARY_SUBSCR
-        subscript2        ::= expr expr DUP_TOP_TWO BINARY_SUBSCR
-
         yield             ::= expr YIELD_VALUE
-
-        expr              ::= if_exp
-
-        compare           ::= compare_chained
-        compare           ::= compare_single
-        compare_single    ::= expr expr COMPARE_OP
-        c_compare         ::= c_compare_chained
-
-
-        # FIXME: the below is to work around test_grammar expecting a "call" to be
-        # on the LHS because it is also somewhere on in a rule.
-        call           ::= expr CALL_METHOD_0
         """
 
-    def p_list_comprehension(self, args):
+    def p_comprehension_list(self, args):
         """
-        expr ::= list_comp
+        lc_body         ::= expr LIST_APPEND
+        list_comp       ::= BUILD_LIST_0 list_iter
 
-        list_iter ::= list_for
-        list_iter ::= list_if
-        list_iter ::= list_if_not
-        list_iter ::= list_if_or_not
-        list_iter ::= lc_body
+        list_iter       ::= list_for
+        list_iter       ::= list_if
+        list_iter       ::= list_if_not
+        list_iter       ::= list_if_or_not
+        list_iter       ::= lc_body
 
-        lc_body   ::= expr LIST_APPEND
-        list_for  ::= expr for_iter store list_iter jb_or_c _come_froms
-        list_comp ::= BUILD_LIST_0 list_iter
+        set_iter        ::= set_for
+        set_iter        ::= list_if
+        # set_iter        ::= list_if_and_or
+        # set_iter        ::= list_if_chained
+        set_iter        ::= list_if_not
+        set_iter        ::= set_comp_body
+
+        list_for  ::= expr_or_arg
+                      for_iter
+                      store list_iter
+                      jb_or_c _come_froms
+
+        set_for   ::= expr_or_arg
+                      for_iter
+                      store set_iter
+                      jb_or_c _come_froms
+
+        list_if_not_end ::= pjump_ift _come_froms
+        list_if_not ::= expr list_if_not_end list_iter come_from_opt
 
         list_if     ::= expr pjump_iff list_iter come_from_opt
-        list_if_not ::= expr pjump_ift list_iter come_from_opt
         list_if     ::= expr jump_if_false_cf   list_iter
         list_if_or_not ::= expr_pjit expr_pjit COME_FROM list_iter
 
-        jb_or_c ::= JUMP_BACK
+        list_if_end ::= pjump_iff _come_froms
+        list_if     ::= expr list_if_end list_iter come_from_opt
+
+        jb_or_c ::= JUMP_LOOP
         jb_or_c ::= CONTINUE
 
 
@@ -389,7 +438,7 @@ class Python37LambdaParser(Python37BaseParser):
 
     def p_37conditionals(self, args):
         """
-        expr                       ::= if_exp37
+        expr                       ::= if_exp_compare
         bool_op                    ::= and_cond
         bool_op                    ::= and_not_cond
         bool_op                    ::= and POP_JUMP_IF_TRUE expr
@@ -404,8 +453,16 @@ class Python37LambdaParser(Python37BaseParser):
 
         if_exp                     ::= expr_pjif expr jump_forward_else expr come_froms
 
-        if_exp37                   ::= expr expr    jf_cfs expr COME_FROM
-        if_exp37                   ::= bool_op expr jf_cfs expr COME_FROM
+        if_exp_compare             ::= expr expr jf_cfs expr COME_FROM
+        if_exp_compare             ::= bool_op expr jf_cfs expr COME_FROM
+
+        if_exp_loop                ::= expr_pjif
+                                       expr
+                                       POP_JUMP_IF_FALSE_LOOP
+                                       JUMP_FORWARD
+                                       come_froms
+                                       expr
+
         jf_cfs                     ::= JUMP_FORWARD _come_froms
         list_iter                  ::= list_if37
         list_iter                  ::= list_if37_not
@@ -426,50 +483,149 @@ class Python37LambdaParser(Python37BaseParser):
         if_exp_37b                 ::= expr_pjif expr_pjif jump_forward_else expr
         """
 
-    def p_comprehension3(self, args):
+    def p_comprehension(self, args):
         """
         # Python3 scanner adds LOAD_LISTCOMP. Python3 does list comprehension like
         # other comprehensions (set, dictionary).
 
-        # Our "continue" heuristic -  in two successive JUMP_BACKS, the first
-        # one may be a continue - sometimes classifies a JUMP_BACK
+        comp_body      ::= dict_comp_body
+        comp_body      ::= gen_comp_body
+        # FIXME: decompile-cfg has this. We are missing a LHS rule?
+        # comp_body      ::= list_comp_body
+        comp_body      ::= set_comp_body
+
+        # Our "continue" heuristic -  in two successive JUMP_LOOPS, the first
+        # one may be a continue - sometimes classifies a JUMP_LOOP
         # as a CONTINUE. The two are kind of the same in a comprehension.
 
-        comp_for       ::= expr get_for_iter store comp_iter CONTINUE _come_froms
-        comp_for       ::= expr get_for_iter store comp_iter JUMP_BACK _come_froms
+        comp_for       ::= expr get_for_iter store comp_iter
+                           CONTINUE
+                           _come_froms
+
+        comp_for       ::= expr get_for_iter store comp_iter
+                           JUMP_LOOP
+                           _come_froms
+
         get_for_iter   ::= GET_ITER _come_froms FOR_ITER
 
-        comp_body      ::= dict_comp_body
-        comp_body      ::= set_comp_body
         dict_comp_body ::= expr expr MAP_ADD
         set_comp_body  ::= expr SET_ADD
 
         # See also common Python p_list_comprehension
-        """
 
-    def p_dict_comp3(self, args):
-        """"
-        or_jump_if_false_cf    ::= or POP_JUMP_IF_FALSE COME_FROM
-        c_or_jump_if_false_cf  ::= c_or POP_JUMP_IF_FALSE_BACK COME_FROM
+        comp_if        ::= expr_pjif comp_iter
+        comp_if         ::= expr_pjiff comp_iter
+        comp_if         ::= c_compare comp_iter
+        comp_if         ::= or_jump_if_false_cf comp_iter
+        comp_if         ::= or_jump_if_false_loop_cf comp_iter
 
-        c_or       ::= or
-        c_or       ::= c_or_parts expr
-        c_or_parts ::= expr_pjift+
+        # We need to have a reduction rule to disambiguate
+        # these "comp_if_not" and "comp_if". The difference is burried in the
+        # sense of the jump in
+        #     comp_iter -> comp_if_or -> or_parts_false_loop
+        # vs.:
+        #    comp_iter -> comp_if_or -> or_parts_true_loop
+        #
+        # If "true_loop then that goes with "comp_if_not"
+        # if "false_loop"  then that goes with comp_if"
+        #
+        # We might be able to do this in the grammar but it is a bit
+        # too pervasive and involved.
+
+        # We have a bunch of these comp_if_<logic expression>
+        # because the logic operation bleeds into the
+        # "if" of the comprehension. Note thet specific position of
+        # POP_JUMP_IF_xxx_LOOP stays the same.
+        comp_if_or      ::= or_parts
+                            expr POP_JUMP_IF_FALSE_LOOP
+                            come_froms
+                            comp_iter
+        # comp_if_or      ::= or_parts_true_loop
+        #                     expr POP_JUMP_IF_FALSE_LOOP
+        #                     come_froms
+        #                     comp_iter
+
+        # comp_if_or      ::= or_parts_false_loop
+        #                     expr POP_JUMP_IF_FALSE_LOOP
+        #                     come_froms
+        #                     comp_iter
+
+        # Here, the "or" is melded a little into the "comp_if" test
+        comp_if_or2     ::= compare compare_chained37_false comp_iter
+
+        comp_if_or_not  ::= or_parts
+                            expr POP_JUMP_IF_TRUE_LOOP
+                            come_froms
+                            comp_iter
+        ## FIXME: we add this, per comment above later.
+        ## comp_if         ::= expr pjump_ift comp_iter
+        comp_if_not     ::= expr pjump_ift comp_iter
+
+
+        comp_if_not_and ::= expr_pjif
+                            expr POP_JUMP_IF_TRUE_LOOP
+                            come_froms
+                            comp_iter
+        comp_if_not_or  ::= expr_pjif
+                            expr POP_JUMP_IF_FALSE_LOOP
+                            come_from_opt
+                            comp_iter
+
+        comp_iter     ::= dict_comp_body
+        comp_iter     ::= comp_body
+        comp_iter     ::= comp_if
+        comp_iter     ::= comp_if_not
+        comp_iter     ::= comp_if_not_and
+        comp_iter     ::= comp_if_not_or
+        comp_iter     ::= comp_if_or
+        comp_iter     ::= comp_if_or_not
+        comp_iter     ::= comp_if_or2
+
+        or_jump_if_false_cf      ::= or POP_JUMP_IF_FALSE COME_FROM
+        or_jump_if_false_loop_cf ::= or_loop POP_JUMP_IF_FALSE_LOOP COME_FROM
+
+        or_loop       ::= or
+        or_loop       ::= or_parts_loop expr
+        or_parts_loop ::= expr_pjift+
 
         # Semantic rules require "comp_if" to have index 0 be some
         # sort of "expr" and index 1 to be some sort of "comp_iter"
         c_compare     ::= compare
 
-        comp_if       ::= expr_pjif comp_iter
-        comp_if       ::= expr_pjiff comp_iter
-        comp_if       ::= c_compare comp_iter
-        comp_if       ::= or_jump_if_false_cf comp_iter
-        comp_if       ::= c_or_jump_if_false_cf comp_iter
-        comp_if_not   ::= expr pjump_ift comp_iter
+        expr_or_arg     ::= LOAD_ARG
+        expr_or_arg     ::= expr
 
-        comp_iter     ::= comp_body
-        comp_iter     ::= comp_if
-        comp_iter     ::= comp_if_not
+        ending_return  ::= RETURN_VALUE RETURN_LAST
+        ending_return  ::= RETURN_VALUE_LAMBDA LAMBDA_MARKER
+
+        for_iter       ::= _come_froms FOR_ITER
+        dict_comp_func ::= BUILD_MAP_0 LOAD_ARG for_iter store
+                           comp_iter JUMP_LOOP _come_froms
+                           ending_return
+
+        set_comp_func   ::= BUILD_SET_0
+                            expr_or_arg
+                            for_iter store comp_iter
+                            JUMP_LOOP
+                            _come_froms
+                            ending_return
+
+        set_comp_func   ::= BUILD_SET_0
+                            expr_or_arg
+                            for_iter store comp_iter
+                            COME_FROM
+                            JUMP_LOOP
+                            _come_froms
+                            ending_return
+
+        await_expr       ::= expr GET_AWAITABLE LOAD_CONST YIELD_FROM
+        set_comp_func    ::= BUILD_SET_0
+                             expr_or_arg
+                             for_iter store await_expr
+                             SET_ADD
+                             JUMP_LOOP
+                             _come_froms
+                             ending_return
         """
 
     def p_expr3(self, args):
@@ -493,8 +649,8 @@ class Python37LambdaParser(Python37BaseParser):
     def p_set_comp(self, args):
         """
         comp_iter     ::= comp_for
-        comp_body     ::= gen_comp_body
         gen_comp_body ::= expr YIELD_VALUE POP_TOP
+        set_comp      ::= BUILD_SET_0 set_iter
         """
 
     def p_store(self, args):
@@ -519,276 +675,6 @@ class Python37LambdaParser(Python37BaseParser):
         store_subscript ::= expr expr STORE_SUBSCR
         """
 
-    def customize_grammar_rules(self, tokens, customize):
-        super(Python37LambdaParser, self).customize_grammar_rules(tokens, customize)
-        self.check_reduce["call_kw"] = "AST"
-
-        for i, token in enumerate(tokens):
-            opname = token.kind
-
-            if opname == "LOAD_ASSERT":
-                if "PyPy" in customize:
-                    rules_str = """
-                    stmt ::= JUMP_IF_NOT_DEBUG stmts COME_FROM
-                    """
-                    self.add_unique_doc_rules(rules_str, customize)
-            elif opname == "FORMAT_VALUE":
-                rules_str = """
-                    expr              ::= formatted_value1
-                    formatted_value1  ::= expr FORMAT_VALUE
-                """
-                self.add_unique_doc_rules(rules_str, customize)
-            elif opname == "FORMAT_VALUE_ATTR":
-                rules_str = """
-                expr              ::= formatted_value2
-                formatted_value2  ::= expr expr FORMAT_VALUE_ATTR
-                """
-                self.add_unique_doc_rules(rules_str, customize)
-            elif opname == "MAKE_FUNCTION_8":
-                if "LOAD_DICTCOMP" in self.seen_ops:
-                    # Is there something general going on here?
-                    rule = """
-                       dict_comp ::= load_closure LOAD_DICTCOMP LOAD_STR
-                                     MAKE_FUNCTION_8 expr
-                                     GET_ITER CALL_FUNCTION_1
-                       """
-                    self.addRule(rule, nop_func)
-                elif "LOAD_SETCOMP" in self.seen_ops:
-                    rule = """
-                       set_comp ::= load_closure LOAD_SETCOMP LOAD_STR
-                                    MAKE_FUNCTION_8 expr
-                                    GET_ITER CALL_FUNCTION_1
-                       """
-                    self.addRule(rule, nop_func)
-
-            elif opname == "BEFORE_ASYNC_WITH":
-                rules_str = """
-                  stmt               ::= async_with_stmt SETUP_ASYNC_WITH
-                  async_with_pre     ::= BEFORE_ASYNC_WITH GET_AWAITABLE LOAD_CONST YIELD_FROM SETUP_ASYNC_WITH
-                  async_with_post    ::= COME_FROM_ASYNC_WITH
-                                         WITH_CLEANUP_START GET_AWAITABLE LOAD_CONST YIELD_FROM
-                                         WITH_CLEANUP_FINISH END_FINALLY
-
-                  stmt               ::= async_with_as_stmt
-                  async_with_as_stmt ::= expr
-                                         async_with_pre
-                                         store
-                                         suite_stmts_opt
-                                         POP_BLOCK LOAD_CONST
-                                         async_with_post
-
-                 async_with_stmt     ::= expr
-                                         async_with_pre
-                                         POP_TOP
-                                         suite_stmts_opt
-                                         POP_BLOCK LOAD_CONST
-                                         async_with_post
-                 async_with_stmt     ::= expr
-                                         async_with_pre
-                                         POP_TOP
-                                         suite_stmts_opt
-                                         async_with_post
-                """
-                self.addRule(rules_str, nop_func)
-
-            elif opname.startswith("BUILD_STRING"):
-                v = token.attr
-                rules_str = """
-                    expr                 ::= joined_str
-                    joined_str           ::= %sBUILD_STRING_%d
-                """ % (
-                    "expr " * v,
-                    v,
-                )
-                self.add_unique_doc_rules(rules_str, customize)
-                if "FORMAT_VALUE_ATTR" in self.seen_ops:
-                    rules_str = """
-                      formatted_value_attr ::= expr expr FORMAT_VALUE_ATTR expr BUILD_STRING
-                      expr                 ::= formatted_value_attr
-                    """
-                    self.add_unique_doc_rules(rules_str, customize)
-            elif opname.startswith("BUILD_MAP_UNPACK_WITH_CALL"):
-                v = token.attr
-                rule = "build_map_unpack_with_call ::= %s%s" % ("expr " * v, opname)
-                self.addRule(rule, nop_func)
-            elif opname.startswith("BUILD_TUPLE_UNPACK_WITH_CALL"):
-                v = token.attr
-                rule = (
-                    "build_tuple_unpack_with_call ::= "
-                    + "expr1024 " * int(v // 1024)
-                    + "expr32 " * int((v // 32) % 32)
-                    + "expr " * (v % 32)
-                    + opname
-                )
-                self.addRule(rule, nop_func)
-                rule = "starred ::= %s %s" % ("expr " * v, opname)
-                self.addRule(rule, nop_func)
-            elif opname == "SETUP_WITH":
-                rules_str = """
-                with       ::= expr SETUP_WITH POP_TOP suite_stmts_opt COME_FROM_WITH
-                               WITH_CLEANUP_START WITH_CLEANUP_FINISH END_FINALLY
-
-                # Removes POP_BLOCK LOAD_CONST from 3.6-
-                withasstmt ::= expr SETUP_WITH store suite_stmts_opt COME_FROM_WITH
-                               WITH_CLEANUP_START WITH_CLEANUP_FINISH END_FINALLY
-                """
-                if self.version < (3, 8):
-                    rules_str += """
-                    with       ::= expr SETUP_WITH POP_TOP suite_stmts_opt POP_BLOCK
-                                   LOAD_CONST
-                                   WITH_CLEANUP_START WITH_CLEANUP_FINISH END_FINALLY
-                    """
-                else:
-                    rules_str += """
-                    with        ::= expr SETUP_WITH POP_TOP suite_stmts_opt POP_BLOCK
-                                   BEGIN_FINALLY COME_FROM_WITH
-                                   WITH_CLEANUP_START WITH_CLEANUP_FINISH
-                                   END_FINALLY
-                    """
-                self.addRule(rules_str, nop_func)
-                pass
-            pass
-
-    def custom_classfunc_rule(self, opname, token, customize, next_token):
-
-        args_pos, args_kw = self.get_pos_kw(token)
-
-        # Additional exprs for * and ** args:
-        #  0 if neither
-        #  1 for CALL_FUNCTION_VAR or CALL_FUNCTION_KW
-        #  2 for * and ** args (CALL_FUNCTION_VAR_KW).
-        # Yes, this computation based on instruction name is a little bit hoaky.
-        nak = (len(opname) - len("CALL_FUNCTION")) // 3
-        uniq_param = args_kw + args_pos
-
-        if frozenset(("GET_AWAITABLE", "YIELD_FROM")).issubset(self.seen_ops):
-            rule = (
-                "async_call ::= expr "
-                + ("expr " * args_pos)
-                + ("kwarg " * args_kw)
-                + "expr " * nak
-                + token.kind
-                + " GET_AWAITABLE LOAD_CONST YIELD_FROM"
-            )
-            self.add_unique_rule(rule, token.kind, uniq_param, customize)
-            self.add_unique_rule(
-                "expr ::= async_call", token.kind, uniq_param, customize
-            )
-
-        if opname.startswith("CALL_FUNCTION_KW"):
-            self.addRule("expr ::= call_kw36", nop_func)
-            values = "expr " * token.attr
-            rule = "call_kw36 ::= expr {values} LOAD_CONST {opname}".format(**locals())
-            self.add_unique_rule(rule, token.kind, token.attr, customize)
-        elif opname == "CALL_FUNCTION_EX_KW":
-            # Note: this doesn't exist in 3.7 and later
-            self.addRule(
-                """expr        ::= call_ex_kw4
-                            call_ex_kw4 ::= expr
-                                            expr
-                                            expr
-                                            CALL_FUNCTION_EX_KW
-                         """,
-                nop_func,
-            )
-            if "BUILD_MAP_UNPACK_WITH_CALL" in self.seen_op_basenames:
-                self.addRule(
-                    """expr        ::= call_ex_kw
-                                call_ex_kw  ::= expr expr build_map_unpack_with_call
-                                                CALL_FUNCTION_EX_KW
-                             """,
-                    nop_func,
-                )
-            if "BUILD_TUPLE_UNPACK_WITH_CALL" in self.seen_op_basenames:
-                # FIXME: should this be parameterized by EX value?
-                self.addRule(
-                    """expr        ::= call_ex_kw3
-                                call_ex_kw3 ::= expr
-                                                build_tuple_unpack_with_call
-                                                expr
-                                                CALL_FUNCTION_EX_KW
-                             """,
-                    nop_func,
-                )
-                if "BUILD_MAP_UNPACK_WITH_CALL" in self.seen_op_basenames:
-                    # FIXME: should this be parameterized by EX value?
-                    self.addRule(
-                        """expr        ::= call_ex_kw2
-                                    call_ex_kw2 ::= expr
-                                                    build_tuple_unpack_with_call
-                                                    build_map_unpack_with_call
-                                                    CALL_FUNCTION_EX_KW
-                             """,
-                        nop_func,
-                    )
-
-        elif opname == "CALL_FUNCTION_EX":
-            self.addRule(
-                """
-                         expr        ::= call_ex
-                         starred     ::= expr
-                         call_ex     ::= expr starred CALL_FUNCTION_EX
-                         """,
-                nop_func,
-            )
-            if "BUILD_MAP_UNPACK_WITH_CALL" in self.seen_ops:
-                self.addRule(
-                    """
-                        expr        ::= call_ex_kw
-                        call_ex_kw  ::= expr expr
-                                        build_map_unpack_with_call CALL_FUNCTION_EX
-                        """,
-                    nop_func,
-                )
-            if "BUILD_TUPLE_UNPACK_WITH_CALL" in self.seen_ops:
-                self.addRule(
-                    """
-                        expr        ::= call_ex_kw3
-                        call_ex_kw3 ::= expr
-                                        build_tuple_unpack_with_call
-                                        %s
-                                        CALL_FUNCTION_EX
-                        """
-                    % "expr "
-                    * token.attr,
-                    nop_func,
-                )
-                pass
-
-            # FIXME: Is this right?
-            self.addRule(
-                """
-                        expr        ::= call_ex_kw4
-                        call_ex_kw4 ::= expr
-                                        expr
-                                        expr
-                                        CALL_FUNCTION_EX
-                        """,
-                nop_func,
-            )
-            pass
-        else:
-            super(Python37LambdaParser, self).custom_classfunc_rule(
-                opname, token, customize, next_token
-            )
-
-    def reduce_is_invalid(self, rule, ast, tokens, first, last):
-        invalid = super(Python37LambdaParser, self).reduce_is_invalid(
-            rule, ast, tokens, first, last
-        )
-        if invalid:
-            return invalid
-        if rule[0] == "call_kw":
-            # Make sure we don't derive call_kw
-            nt = ast[0]
-            while not isinstance(nt, Token):
-                if nt[0] == "call_kw":
-                    return True
-                nt = nt[0]
-                pass
-            pass
-        return False
-
 
 if __name__ == "__main__":
     # Check grammar
@@ -796,10 +682,10 @@ if __name__ == "__main__":
 
     p = Python37LambdaParser()
     modified_tokens = set(
-        """JUMP_BACK CONTINUE RETURN_END_IF_LAMBDA COME_FROM
+        """JUMP_LOOP CONTINUE RETURN_END_IF_LAMBDA COME_FROM
            LOAD_GENEXPR LOAD_ASSERT LOAD_SETCOMP LOAD_DICTCOMP LOAD_CLASSNAME
            LAMBDA_MARKER RETURN_VALUE_LAMBDA
         """.split()
     )
 
-    dump_and_check(p, 3.7, modified_tokens)
+    dump_and_check(p, (3, 7), modified_tokens)
