@@ -21,13 +21,11 @@ scanner/ingestion module. From here we call various version-specific
 scanners, e.g. for Python 3.7 or 3.8.
 """
 
+import xdis
 from typing import Optional
 from array import array
 from collections import namedtuple
-
 from xdis.version_info import IS_PYPY, version_tuple_to_str
-from decompyle3.scanners.tok import Token
-import xdis
 from xdis import (
     Bytecode,
     canonic_python_version,
@@ -37,6 +35,8 @@ from xdis import (
     next_offset,
 )
 
+from decompyle3.scanners.tok import Token
+
 # The byte code versions we support.
 # Note: these all have to be floats
 PYTHON_VERSIONS = frozenset(((3, 7), (3, 8), (3, 9)))
@@ -61,11 +61,11 @@ class Code(object):
     the diassembled code is stored in the attribute '_tokens'.
     """
 
-    def __init__(self, co, scanner, classname=None):
+    def __init__(self, co, scanner, classname=None, show_asm=None):
         for i in dir(co):
             if i.startswith("co_"):
                 setattr(self, i, getattr(co, i))
-        self._tokens, self._customize = scanner.ingest(co, classname)
+        self._tokens, self._customize = scanner.ingest(co, classname, show_asm=show_asm)
 
 
 class Scanner(object):
@@ -74,14 +74,17 @@ class Scanner(object):
         self.show_asm = show_asm
         self.is_pypy = is_pypy
 
-        if version in PYTHON_VERSIONS:
+        if version[:2] in PYTHON_VERSIONS:
             v_str = f"""opcode_{version_tuple_to_str(version, start=0, end=2, delimiter="")}"""
             if is_pypy:
                 v_str += "pypy"
             exec(f"""from xdis.opcodes import {v_str}""")
             exec("self.opc = %s" % v_str)
         else:
-            raise TypeError("%s is not a Python version I know about" % version)
+            raise TypeError(
+                "%s is not a Python version I know about"
+                % version_tuple_to_str(version)
+            )
 
         self.opname = self.opc.opname
 
@@ -181,9 +184,31 @@ class Scanner(object):
         # we assume that it was an instruction moved back.
         # We check that assumption though by looking at
         # self.code's opcode.
+        # Sadly instructions can get moved _forward too.
+        # So we have to check which direction we are going
+        offset_increment = instruction_size(self.opc.EXTENDED_ARG, self.opc)
         if offset not in self.offset2inst_index:
-            offset -= instruction_size(self.opc.EXTENDED_ARG, self.opc)
-            assert self.code[offset] == self.opc.EXTENDED_ARG
+            if self.code[offset] != self.opc.EXTENDED_ARG:
+                target_name = self.opc.opname[self.code[offset]]
+                # JUMP_ABSOLUTE can be like this where
+                # the inst offset is at what used to be an EXTENDED_ARG
+                # so find the first extended arg.
+                next_offset = offset - offset_increment
+                while next_offset not in self.offset2inst_index:
+                    next_offset -= offset_increment
+                    assert self.code[next_offset] == self.opc.EXTENDED_ARG
+                inst = self.insts[self.offset2inst_index[next_offset]]
+                assert inst.opname == target_name, inst
+            else:
+                next_offset = offset + offset_increment
+                while next_offset not in self.offset2inst_index:
+                    next_offset += offset_increment
+
+                inst = self.insts[self.offset2inst_index[next_offset]]
+
+            assert inst.has_extended_arg is True
+            return inst
+
         return self.insts[self.offset2inst_index[offset]]
 
     def get_target(self, offset: int, extended_arg: int = 0) -> int:
@@ -316,7 +341,7 @@ class Scanner(object):
         """
         try:
             None in instr
-        except:
+        except Exception:
             instr = [instr]
 
         first = self.offset2inst_index[start]
@@ -494,12 +519,11 @@ def get_scanner(version, is_pypy=False, show_asm=None):
     # If version is a string, turn that into the corresponding float.
     if isinstance(version, str):
         if version not in canonic_python_version:
-            raise RuntimeError("Unknown Python version in xdis %s" % version)
+            raise RuntimeError(f"Unknown Python version in xdis {version}")
         canonic_version = canonic_python_version[version]
         if canonic_version not in CANONIC2VERSION:
             raise RuntimeError(
-                "Unsupported Python version %s (canonic %s)"
-                % (version, canonic_version)
+                f"Unsupported Python version {version} (canonic {canonic_version})"
             )
         version = CANONIC2VERSION[canonic_version]
 
@@ -538,14 +562,16 @@ def get_scanner(version, is_pypy=False, show_asm=None):
             )
     else:
         raise RuntimeError(
-            f"Unsupported Python version {version_tuple_to_str(version)}"
+            f"Unsupported Python version, {version_tuple_to_str(version)}, for decompilation"
         )
     return scanner
 
 
 if __name__ == "__main__":
-    import inspect, decompyle3
+    import inspect
 
     co = inspect.currentframe().f_code
-    scanner = get_scanner(decompyle3.PYTHON_VERSION, IS_PYPY, True)
+    from xdis.version_info import PYTHON_VERSION_TRIPLE
+
+    scanner = get_scanner(PYTHON_VERSION_TRIPLE, IS_PYPY, True)
     tokens, customize = scanner.ingest(co, {}, show_asm="after")
