@@ -1,4 +1,4 @@
-#  Copyright (c) 2015-2021 by Rocky Bernstein
+#  Copyright (c) 2015-2022 by Rocky Bernstein
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -63,10 +63,9 @@ The node position 0 will be associated with "import".
 
 # FIXME: DRY code with pysource
 
-from __future__ import print_function
-
 import re
 
+import decompyle3.parsers.parse_heads as heads
 import decompyle3.parsers.main as python_parser
 from decompyle3.semantics import pysource
 from decompyle3.scanner import Token, Code, get_scanner
@@ -342,9 +341,10 @@ class FragmentsWalker(pysource.SourceWalker, object):
         else:
             start = len(self.f.getvalue()) + len(self.indent)
             self.write(self.indent, "return")
-            if self.return_none or node != SyntaxTree(
-                "return", [SyntaxTree("ret_expr", [NONE]), Token("RETURN_VALUE")]
-            ):
+            # One reason we worry over whether we use "return None" or "return"
+            # is that inside a generator, "return None" is illegal.
+            # Thank you, Python!
+            if self.return_none or not self.is_return_none(node):
                 self.write(" ")
                 self.last_finish = len(self.f.getvalue())
                 self.preorder(node[0])
@@ -370,9 +370,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
         else:
             start = len(self.f.getvalue()) + len(self.indent)
             self.write(self.indent, "return")
-            if self.return_none or node != SyntaxTree(
-                "return", [SyntaxTree("ret_expr", [NONE]), Token("RETURN_END_IF")]
-            ):
+            if self.return_none or not self.is_return_none(node):
                 self.write(" ")
                 self.preorder(node[0])
                 if hasattr(node[-1], "offset"):
@@ -454,9 +452,9 @@ class FragmentsWalker(pysource.SourceWalker, object):
         self.set_pos_info(node, start, len(self.f.getvalue()))
         self.prune()
 
-    def n_ret_expr(self, node):
+    def n_return_expr(self, node):
         start = len(self.f.getvalue())
-        super(FragmentsWalker, self).n_ret_expr(node)
+        super(FragmentsWalker, self).n_return_expr(node)
         self.set_pos_info(node, start, len(self.f.getvalue()))
 
     def n_bin_op(self, node):
@@ -675,7 +673,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
             elif n == "comp_if_not":
                 n = n[2]
 
-        assert n == "comp_body", ast
+        assert n == "comp_body", n
 
         self.preorder(n[0])
         if node == "generator_exp_async":
@@ -716,7 +714,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
 
         assert iscode(code), node[code_index]
         code_name = code.co_name
-        code = Code(code, self.scanner, self.currentclass)
+        code = Code(code, self.scanner, self.currentclass, self.debug_opts["asm"])
 
         ast = self.build_ast(code._tokens, code._customize, code)
 
@@ -724,7 +722,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
         if ast[0] == "sstmt":
             ast = ast[0]
 
-        # skip over stmt return ret_expr
+        # skip over stmt return return_expr
         ast = ast[0][0][0]
         store = None
         if ast in ["set_comp_func", "dict_comp_func"]:
@@ -908,7 +906,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
             self.set_pos_info(node[0], start - 1, start)
             self.comprehension_walk3(node, 1, 0)
         elif node[0].kind == "load_closure":
-            self.setcomprehension_walk3(node, collection_index=4)
+            self.closure_walk(node, collection_index=4)
         else:
             self.comprehension_walk(node, iter_index=4)
         self.write("}")
@@ -964,11 +962,8 @@ class FragmentsWalker(pysource.SourceWalker, object):
         self.write("]")
         self.prune()
 
-    def setcomprehension_walk3(self, node, collection_index):
-        """Set comprehensions the way they are done in Python3.
-        They're more other comprehensions, e.g. set comprehensions
-        See if we can combine code.
-        """
+    def closure_walk(self, node, collection_index):
+        """Dictionary and Set comprehensions using closures."""
         p = self.prec
         self.prec = 27
 
@@ -1107,13 +1102,22 @@ class FragmentsWalker(pysource.SourceWalker, object):
 
     n_classdefdeco2 = n_classdef
 
-    def gen_source(self, ast, name, customize, is_lambda=False, returnNone=False):
+    def gen_source(
+        self,
+        ast,
+        name,
+        customize,
+        is_lambda=False,
+        returnNone=False,
+        debug_opts=pysource.DEFAULT_DEBUG_OPTS,
+    ):
         """convert parse tree to Python source code"""
 
         rn = self.return_none
         self.return_none = returnNone
         old_name = self.name
         self.name = name
+        self.debug_opts = debug_opts
         # if code would be empty, append 'pass'
         if len(ast) == 0:
             self.println(self.indent, "pass")
@@ -1130,7 +1134,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
         code,
         is_lambda=False,
         noneInNames=False,
-        isTopLevel=False,
+        is_top_level_module=False,
     ):
 
         # FIXME: DRY with pysource.py
@@ -1153,16 +1157,16 @@ class FragmentsWalker(pysource.SourceWalker, object):
                 self.p.opc = self.scanner.opc
                 ast = python_parser.parse(self.p, tokens, customize, is_lambda)
                 self.p.insts = p_insts
-            except (python_parser.ParserError, AssertionError) as e:
+            except (heads.ParserError, AssertionError) as e:
                 raise ParserError(e, tokens, self.debug_parser.get("reduce", False))
 
-            ## FIXME: So as not to remove tokens with offsets,
-            ## remove this phase until we have a chance to go over,
-            # transform_ast = self.treeTransform.transform(ast)
+            # FIXME: So as not to remove tokens with offsets,
+            # remove this phase until we have a chance to go over,
+            # transform_tree = self.treeTransform.transform(ast)
             maybe_show_tree(self, ast)
             return ast
             # del ast # Save memory
-            # return transform_ast
+            # return transform_tree
 
         # The bytecode for the end of the main routine has a
         # "return None". However you can't issue a "return" statement in
@@ -1179,7 +1183,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
                 # Python 3.4's classes can add a "return None" which is
                 # invalid syntax.
                 if tokens[-2].kind == "LOAD_CONST":
-                    if isTopLevel or tokens[-2].pattr is None:
+                    if is_top_level_module or tokens[-2].pattr is None:
                         del tokens[-2:]
                     else:
                         tokens.append(Token("RETURN_LAST"))
@@ -1196,7 +1200,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
             self.p.insts = self.scanner.insts
             ast = python_parser.parse(self.p, tokens, customize, is_lambda=is_lambda)
             self.p.insts = p_insts
-        except (python_parser.ParserError, AssertionError) as e:
+        except (heads.ParserError, AssertionError) as e:
             raise ParserError(e, tokens, self.debug_parser.get("reduce", False))
 
         maybe_show_tree(self, ast)
@@ -1706,12 +1710,12 @@ class FragmentsWalker(pysource.SourceWalker, object):
                     if isinstance(index[1], str):
                         assert node[index[0]] == index[1], (
                             "at %s[%d], expected %s node; got %s"
-                            % (node.kind, arg, node[index[0]].kind, index[1])
+                            % (node.kind, arg, node[index[0]].kind, index[1],)
                         )
                     else:
                         assert node[index[0]] in index[1], (
                             "at %s[%d], expected to be in '%s' node; got '%s'"
-                            % (node.kind, arg, index[1], node[index[0]].kind)
+                            % (node.kind, arg, index[1], node[index[0]].kind,)
                         )
 
                     index = index[0]
@@ -1732,7 +1736,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
                     (index, nonterm_name, self.prec) = tup
                     assert node[index] == nonterm_name, (
                         "at %s[%d], expected '%s' node; got '%s'"
-                        % (node.kind, arg, nonterm_name, node[index].kind)
+                        % (node.kind, arg, nonterm_name, node[index].kind,)
                     )
                 else:
                     assert len(tup) == 2
@@ -1934,8 +1938,10 @@ def code_deparse(
         is_pypy=is_pypy,
     )
 
-    isTopLevel = co.co_name == "<module>"
-    deparsed.ast = deparsed.build_ast(tokens, customize, co, isTopLevel=isTopLevel)
+    is_top_level_module = co.co_name == "<module>"
+    deparsed.ast = deparsed.build_ast(
+        tokens, customize, co, is_top_level_module=is_top_level_module
+    )
 
     assert deparsed.ast == "stmts", "Should have parsed grammar start"
 
@@ -2008,7 +2014,7 @@ def code_deparse_around_offset(
     assert iscode(co)
 
     if version is None:
-        version = sysinfo2float()
+        version = PYTHON_VERSION_TRIPLE
     if is_pypy is None:
         is_pypy = IS_PYPY
 
@@ -2061,8 +2067,7 @@ def deparsed_find(tup, deparsed, code):
     """Return a NodeInfo nametuple for a fragment-deparsed `deparsed` at `tup`.
 
     `tup` is a name and offset tuple, `deparsed` is a fragment object
-    and `code` is instruction bytecode.
-"""
+    and `code` is instruction bytecode."""
     nodeInfo = None
     name, last_i = tup
     if not hasattr(deparsed, "offsets"):
