@@ -1,5 +1,5 @@
 import re
-from decompyle3 import PYTHON_VERSION_TRIPLE, IS_PYPY
+from xdis import PYTHON_VERSION_TRIPLE, IS_PYPY
 from decompyle3.parsers.main import get_python_parser, python_parser
 
 
@@ -20,7 +20,8 @@ def test_grammar():
 
     expect_lhs = set([])
 
-    # We have custom rules that create the below
+    # We have custom rules that create the below.
+    # await_expr can be remove after we create a p37/full_custom.py
     unused_rhs = set(["mkfunc"])
 
     expect_right_recursive = set([("designList", ("store", "DUP_TOP", "designList"))])
@@ -49,7 +50,7 @@ def test_grammar():
             ("CONTINUE",),
             ("JUMP_ABSOLUTE",),
             ("LOAD_CONST",),
-            ("JUMP_BACK",),
+            ("JUMP_LOOP",),
             ("JUMP_FORWARD",),
         ]
     )
@@ -64,7 +65,7 @@ def test_grammar():
 
     # ignore_set = set(
     #     """
-    #         JUMP_BACK CONTINUE
+    #         JUMP_LOOP CONTINUE
     #         COME_FROM COME_FROM_EXCEPT
     #         COME_FROM_EXCEPT_CLAUSE
     #         COME_FROM_LOOP COME_FROM_WITH
@@ -80,8 +81,8 @@ def test_dup_rule():
     import inspect
 
     python_parser(
-        PYTHON_VERSION_TRIPLE,
         inspect.currentframe().f_code,
+        PYTHON_VERSION_TRIPLE,
         is_pypy=IS_PYPY,
         parser_debug={
             "dups": True,
