@@ -21,7 +21,7 @@ Step 1) Edit this file and add a new entry to 'test_options', eg.
   test_options['mylib'] = ('/usr/lib/mylib', PYOC, 'mylib')
 Step 2: Run the test:
   test_pythonlib.py --mylib	  # decompile 'mylib'
-  test_pythonlib.py --mylib --syntaix-verify # decompile verify 'mylib'
+  test_pythonlib.py --mylib --syntax-verify # decompile verify 'mylib'
 """
 
 import getopt, os, py_compile, sys, shutil, tempfile, time
@@ -61,7 +61,7 @@ for vers in (3.7, 3.8):
     key = "bytecode-%s" % vers
     test_options[key] = (bytecode, PYC, bytecode, vers)
     bytecode = "bytecode_%s_run" % vers
-    key = "bytecode-%s-run" % vers
+    key = "bytecode-%s/run" % vers
     test_options[key] = (bytecode, PYC, bytecode, vers)
     key = "%s" % vers
     pythonlib = "python%s" % vers
@@ -69,6 +69,12 @@ for vers in (3.7, 3.8):
         pythonlib = os.path.join(pythonlib, "__pycache__")
     test_options[key] = (os.path.join(lib_prefix, pythonlib), PYOC, pythonlib, vers)
 
+vers = 3.7
+bytecode = "bytecode_pypy%s_run" % vers
+key = "bytecode-pypy37"
+test_options[key] = (bytecode, PYC, bytecode, vers)
+
+
 # -----
 
 
@@ -98,6 +104,20 @@ def do_tests(src_dir, obj_patterns, target_dir, opts):
         )
 
     files = []
+
+    if opts["compile_type"] == "lambda":
+        src_dir += "/code-fragment/lambda"
+    elif opts["compile_type"] == "dict-comprehension":
+        src_dir += "/code-fragment/dict-comprehension"
+    elif opts["compile_type"] == "list-comprehension":
+        src_dir += "/code-fragment/list-comprehension"
+    elif opts["compile_type"] == "set-comprehension":
+        src_dir += "/code-fragment/set-comprehension"
+    elif opts["compile_type"] == "run":
+        src_dir += "/run"
+    else:
+        src_dir += "/exec"
+
     # Change directories so use relative rather than
     # absolute paths. This speeds up things, and allows
     # main() to write to a relative-path destination.
@@ -155,7 +175,9 @@ def do_tests(src_dir, obj_patterns, target_dir, opts):
         )
         if failed_files != 0:
             sys.exit(2)
-        elif failed_verify != 0:
+        elif failed_verify:
+            parent_dir = os.path.dirname(target_dir)
+            print("Verify failed, keeping %s" % parent_dir)
             sys.exit(3)
 
     except (KeyboardInterrupt, OSError):
@@ -182,9 +204,15 @@ if __name__ == "__main__":
             "verify-run",
             "syntax-verify",
             "all",
+            "dict-comprehension",
+            "generator",
+            "lambda",
+            "list-comprehension",
+            "set-comprehension",
             "compile",
             "coverage",
             "no-rm",
+            "run",
         ]
         + test_options_keys,
     )
@@ -193,10 +221,11 @@ if __name__ == "__main__":
 
     test_opts = {
         "do_compile": False,
-        "do_verify": False,
+        "do_verify": None,
         "start_with": None,
         "rmtree": True,
         "coverage": False,
+        "compile_type": "exec",
     }
 
     for opt, val in opts:
@@ -206,6 +235,18 @@ if __name__ == "__main__":
             test_opts["do_verify"] = "verify-run"
         elif opt == "--compile":
             test_opts["do_compile"] = True
+        elif opt == "--lambda":
+            test_opts["compile_type"] = "lambda"
+        elif opt == "--dict-comprehension":
+            test_opts["compile_type"] = "dict-comprehension"
+        elif opt == "--generator":
+            test_opts["compile_type"] = "generator"
+        elif opt == "--list-comprehension":
+            test_opts["compile_type"] = "list-comprehension"
+        elif opt == "--run":
+            test_opts["compile_type"] = "run"
+        elif opt == "--set-comprehension":
+            test_opts["compile_type"] = "set-comprehension"
         elif opt == "--start-with":
             test_opts["start_with"] = val
         elif opt == "--no-rm":
